import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class Pez here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public abstract class Pez extends Actor
{
    /** Altura máxima en pixeles que puede alcanzar un pez */
    public static final int ALTURA_MAXIMA = 150;
    
    /**
     * Cantidad de niveles de crecimiento que puede tener un pez. Cada vez que ha comido suficiente,
     * pasa a un siguiente nivel de crecimiento lo cual se refleja en el juego con una imagen de mayor
     * tamaño */
    public static final int NIVELES_CRECIMIENTO = 10;
    
    /**
     * Hay imágenes del pez para cuando está en dirección hacia la derecha, como hacia la izquierda.
     * Estas constantes ayudan a determinar cuáles de ellas se están usando
     */
    public static final int DERECHA = 0;
    public static final int IZQUIERDA = 1;
    
    /**
     * Las imágenes correspondientes a cada uno de los niveles de crecimiento del pez, donde la
     * imagen más pequeña está en la posición 0 y la más grande en NIVELES_CRECIMIENTO - 1
     * Se guardan en una matriz, donde la primera fila tiene las imágenes para cuando el pez
     * está orientado hacia la derecha, y la segunda fila para cuando el pez está en dirección
     * hacia la izquierda. Las columnas representan los diferentes niveles de crecimiento que
     * puede tener el pez. En cada celda se almacena una imagen. Por ejemplo, imagen[1][3] es
     * la imagen del pez hacia la izquierda cuando haya crecido por cuarta vez
     */
    protected GreenfootImage imagenes[][];
    
    /**
     * Indica el nivel de crecimiento actual de este pez. Entre más coma, mas crecerá.
     */
    protected int nivelCrecimiento = 0;
    
    /**
     * Indica la dirección actual del pez, que puede ser una de las constantes DERECHA o IZQUIERDA
     */
    protected int direccion;
    
    /**
     * Crea un nuevo pez con la imagen cuyo nombre de archivo es dado por parámetro. Se asume que en la
     * carpeta de imágenes hay dos imágenes para cada pez. Una terminada en "i" que indica que el pez
     * está mirando hacia la izquierda. La otra terminada en "d" con el pez mirando hacia la derecha.
     * <p>
     * Por ejemplo si en imagen se envia "Pez02", se asume que en la carpeta de imagenes hay dos
     * archivos: "Pez02i.png" y "Pez02d.png".
     * <p>
     * El tamaño de la imagen debe ser de cualquier ancho, pero de altura 150px que para efectos del
     * juego se considera el tamaño máximo que puede alcanzar un pez.
     * <p>
     * El parámetro direccion debe ser 0 si se quiere que el pez a crear esté mirando hacia la
     * derecha (y por tanto usará la imagen terminada en "d"). Si se envía un 1, el pez mirará hacia
     * la izquierda (y por tanto la imagen terminada en "i"). En lugar de los números 0 ó 1, puede
     * enviar las constantes Pez.DERECHA o Pez.IZQUIERDA.
     * <p>
     * El parámetro nivelCrecimiento indica el nivel de crecimiento del nuevo pez. Es un número entre
     * 0 y 9 inclusive. 0 indica el menor tamaño posible, y 9 el máximo.
     */
    public Pez(String imagen, int direccion, int nivelCrecimiento)
    {
        imagenes = new GreenfootImage[2][NIVELES_CRECIMIENTO];
        cargarImagenes(imagen, DERECHA, "d");
        cargarImagenes(imagen, IZQUIERDA, "i");
        this.direccion = direccion;
        this.nivelCrecimiento = nivelCrecimiento;
        setImage( imagenes[direccion][nivelCrecimiento] );
    }
    
    /**
     * Carga la imagen cuyo nombre es dado y construye un conjunto de imágenes de diferente tamaño
     * correspondientes a cada uno de los niveles de crecimiento del pez
     */
    private void cargarImagenes(String imagen, int direccion, String sufijo)
    {
        // De un nivel de crecimiento al siguiente, el pez aumenta esta cantidad de pixeles
        int alturaCrecimiento = ALTURA_MAXIMA / NIVELES_CRECIMIENTO;
        // Construye el nombre completo de la imagen, algo como "Pez00d.png"
        imagen = imagen + sufijo + ".png";
        // Carga la imagen base, de la cual se haran muchas imagenes de diferente tamaño, acordes
        // a cada nivel de crecimiento
        GreenfootImage imagenBase = new GreenfootImage(imagen);
        // Crear una imagen para cada nivel de crecimiento y para cada dirección: derecha/izquierda
        for ( int i = 0; i < NIVELES_CRECIMIENTO; ++i )
        {
            // El alto que debe tener el pez en pixeles para el nivel de crecimiento i
            int alto = (i + 1) * alturaCrecimiento;
            // El ancho se calcula por regla de 3, en proporción al alto
            int ancho = imagenBase.getWidth() * alto / imagenBase.getHeight();
            // Crear la imagen redimensionándola a partir de la imagen base
            imagenes[direccion][i] = new GreenfootImage(imagenBase);
            imagenes[direccion][i].scale( ancho, alto );
        }
    }
    
    /** Permite conocer el nivel de crecmiento de este pez **/
    public int nivelCrecimiento()
    {
        return nivelCrecimiento;
    }
    
    /** Permite conocer la orientación de este pez, 0 indica derecha y 1 indica izquierda **/
    public int direccion()
    {
        return direccion;
    }
    
    /**
     * Incrementa el tamaño del pez al siguiente nivel de crecimiento. El cambio se ve reflejado
     * de inmediato en la imagen. Si el pez ya tiene el máximo nivel de crecimiento, no se hace
     * ningún cambio.
     */
    public void crecer()
    {
        if ( nivelCrecimiento < NIVELES_CRECIMIENTO - 1 )
            setImage( imagenes[direccion][++nivelCrecimiento] );
    }
    
    /**
     * Indica la cantidad de pixeles que se puede mover el pez en cada acto. Entre más pequeño
     * más rápido se mueve, entre más grande, más lento.
     */
    public int pasos()
    {
        return (NIVELES_CRECIMIENTO - nivelCrecimiento) / 3 + 1;
    }
    
    /**
     * Cambia la dirección del pez a la dada. Si es necesario, actualiza la imagen para reflejar
     * la nueva orientación.
     * 
     * @param nuevaDireccion Un entero entre 0 y 1, donde 0 indica derecha y 1 izquierda.
     * Utilice las constantes Pez.DERECHA y Pez.IZQUIERDA
     */
    public void cambiarDireccion(int nuevaDireccion)
    {
        if ( direccion != nuevaDireccion )
        {
            setImage( imagenes[nuevaDireccion][nivelCrecimiento] );
            direccion = nuevaDireccion;
        }
    }
    
    /**
     * Retorna true si el pez ha alcanzado el otro extremo del océano. Esto se determina de la
     * siguiente forma. Un pez que nada hacia la derecha, habrá alcanzado el otro extremo cuando su
     * posición sea el borde derecho del océano. Un pez que nada hacia la izquierda ha alcanzado el
     * otro extremo cuando se encuentre en el borde izquierdo.
     */
    public boolean llegoAlOtroExtremo()
    {
        if ( direccion == DERECHA && getX() >= getWorld().getWidth() - getImage().getWidth() / 2 )
            return true;
        
        if ( direccion == IZQUIERDA && getX() <= getImage().getWidth() / 2 )
            return true;
            
        return false;
    }
    
    /**
     * Act - do whatever the Pez wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
    }
}
